@@ -29,12 +29,12 @@ use crate::{
         TorchDeviceDtype, TorchDeviceDtypeOp,
     },
     util::dict_to_list,
-    util::{hashmap_collect_except_duplicates, EinInt},
+    util::{hashmap_collect_except_duplicates, AxisInt},
 };
 
 // we could use u64 instead of u8 if we wanted.
 // this is u8, so we want to use multiple of 8 inline so we don't pad
-pub type EinsumAxes = Sv<[EinInt; 8]>;
+pub type EinsumAxes = Sv<[AxisInt; 8]>;
 
 /// our Einsum supports diag output
 /// this means `a->aa` is valid and produces a tensor with the input on the diagonal
@@ -71,7 +71,7 @@ impl CircuitNode for Einsum {
     }
 
     fn child_axis_map(&self) -> Vec<Vec<Option<usize>>> {
-        let out_inv: HashMap<EinInt, usize> = hashmap_collect_except_duplicates(
+        let out_inv: HashMap<AxisInt, usize> = hashmap_collect_except_duplicates(
             self.out_axes.iter().enumerate().map(|(i, x)| (*x, i)),
         );
         self.input_axes()
@@ -156,7 +156,7 @@ impl Einsum {
         Box::new(self.args.iter().map(|(_, axes)| axes))
     }
 
-    pub fn shape_map(&self) -> Result<HashMap<EinInt, usize>, CircuitConstructionError> {
+    pub fn shape_map(&self) -> Result<HashMap<AxisInt, usize>, CircuitConstructionError> {
         let mut out = HashMap::new();
         for (circ, axes) in &self.args {
             if circ.info().rank() != axes.len() {
@@ -215,7 +215,8 @@ impl Einsum {
     ) -> Result<Self, CircuitConstructionError> {
         // TODO: maybe check int sizes!
         assert_eq!(spec.input_ints.len(), circuits.len());
-        let to_idx = |vals: &[usize]| -> EinsumAxes { vals.iter().map(|&x| x as EinInt).collect() };
+        let to_idx =
+            |vals: &[usize]| -> EinsumAxes { vals.iter().map(|&x| x as AxisInt).collect() };
         Self::try_new(
             circuits
                 .iter()
@@ -227,7 +228,7 @@ impl Einsum {
         )
     }
 
-    pub fn axes_in_input(&self) -> HashSet<EinInt> {
+    pub fn axes_in_input(&self) -> HashSet<AxisInt> {
         // could be 256 bit bitmask
         self.input_axes().flatten().copied().collect()
     }
@@ -419,6 +420,41 @@ impl Einsum {
         .unwrap()
     }
     #[staticmethod]
+    pub fn elementwise_broadcasted(
+        nodes: Vec<CircuitRc>,
+        name: Option<String>,
+    ) -> Result<Einsum, CircuitConstructionError> {
+        let out_shape = broadcast_shapes(&nodes.iter().map(|x| x.info().shape.clone()).collect())
+            .map_err(|e| CircuitConstructionError::TensorInvariantError { err: e })?;
+        let mut prev_one_shape = out_shape.len().saturating_sub(1) as u8;
+        Einsum::try_new(
+            nodes
+                .iter()
+                .map(|node| {
+                    let rank_difference = out_shape.len() - node.info().rank();
+                    (
+                        node.clone(),
+                        node.info()
+                            .shape
+                            .iter()
+                            .enumerate()
+                            .map(|(i, l)| {
+                                if *l != out_shape[i + rank_difference] {
+                                    prev_one_shape += 1;
+                                    prev_one_shape
+                                } else {
+                                    (i + rank_difference) as u8
+                                }
+                            })
+                            .collect(),
+                    )
+                })
+                .collect(),
+            (0u8..out_shape.len() as u8).collect(),
+            name,
+        )
+    }
+    #[staticmethod]
     pub fn empty(name: Option<String>) -> Self {
         Einsum::try_new(vec![], sv![], name).unwrap()
     }
@@ -882,7 +918,7 @@ impl Rearrange {
 pub struct Index {
     #[pyo3(get)]
     pub node: CircuitRc,
-    #[pyo3(get)]
+    #[pyo3(get, name = "idx")]
     pub index: TensorIndex,
     info: CachedCircuitInfo,
     name: Option<String>,
@@ -1259,7 +1295,7 @@ impl Concat {
 pub struct Scatter {
     #[pyo3(get)]
     pub node: CircuitRc,
-    #[pyo3(get)]
+    #[pyo3(get, name = "idx")]
     pub index: TensorIndex,
     info: CachedCircuitInfo,
     name: Option<String>,
