@@ -117,7 +117,7 @@ impl ArrayConstant {
     #[staticmethod]
     pub fn new_named_axes(value: Tensor, name: Option<String>, named_axes: NamedAxes) -> Self {
         let result = Self::new(value, name);
-        set_named_axes(&result, named_axes)
+        set_named_axes(result, named_axes)
     }
 
     #[staticmethod]
