@@ -2,6 +2,7 @@
 use crate::all_imports::{RInnerInts, Sv, TensorInvariantError};
 use crate::circuit::circuit_utils::toposort_circuit;
 use crate::hashmaps::{AHashSet as HashSet, FxHashMap as HashMap};
+use crate::pyo3_prelude::*;
 use crate::util::AxisInt;
 use crate::{
     cached_lambda,
@@ -12,7 +13,7 @@ use crate::{
     },
     util::AsOp,
 };
-use crate::{pyo3_prelude::*, sv};
+use crate::{pycall, pycallable, sv};
 pub use computational_node::{
     flat_concat, flat_concat_back, Add, Concat, Einsum, EinsumAxes, Index, Rearrange, Scatter,
 };
@@ -97,6 +98,8 @@ mod circuit_node_private {
 }
 use circuit_node_private::*;
 
+use self::circuit_utils::total_arrayconstant_size;
+
 impl<T: CircuitNodePrivate + CircuitNode> CircuitNodeInit for T {
     fn init_info_impl(mut self) -> Result<Self, CircuitConstructionError>
     where
@@ -1237,11 +1240,20 @@ pub enum CircuitConstructionError {
 
     #[error("python error {py_err:?}")]
     PythonError { py_err: Arc<PyErr> },
+
+    #[error("just stopping iteration, not an actual error")]
+    StopIteration {},
 }
 
 impl From<CircuitConstructionError> for PyErr {
     fn from(err: CircuitConstructionError) -> Self {
-        PyErr::new::<exceptions::PyValueError, _>(format!("error (TODO: better) {}", err))
+        match err {
+            CircuitConstructionError::PythonError { py_err } => Python::with_gil(|py| {
+                let err = (*py_err).value(py);
+                err.into()
+            }),
+            _ => PyErr::new::<exceptions::PyValueError, _>(format!("error (TODO: better) {}", err)),
+        }
     }
 }
 
@@ -1322,9 +1334,11 @@ pub fn evaluate_fn_dtype_device(
     recurse(circ)
 }
 
+#[apply(pycallable)]
+#[pyo3(name = "deep_map")]
 pub fn deep_map<F>(circuit: CircuitRc, f: F) -> Result<CircuitRc, CircuitConstructionError>
 where
-    F: Fn(CircuitRc) -> Result<CircuitRc, CircuitConstructionError>,
+    F: Fn((circuit, CircuitRc)) -> Result<CircuitRc, _>,
 {
     #[apply(cached_lambda)]
     #[key(circ.info().hash, HashBytes)]
@@ -1335,9 +1349,11 @@ where
     recurse(circuit)
 }
 
+#[apply(pycallable)]
+#[pyo3(name = "deep_map_preorder")]
 pub fn deep_map_preorder<F>(circuit: CircuitRc, f: F) -> Result<CircuitRc, CircuitConstructionError>
 where
-    F: Fn(CircuitRc) -> Result<CircuitRc, CircuitConstructionError>,
+    F: Fn((circuit, CircuitRc)) -> Result<CircuitRc, _>,
 {
     #[apply(cached_lambda)]
     #[key(circ.info().hash, HashBytes)]
@@ -1347,54 +1363,6 @@ where
     recurse(circuit)
 }
 
-pub fn deep_map_unwrap<F>(circuit: CircuitRc, f: F) -> CircuitRc
-where
-    F: Fn(CircuitRc) -> CircuitRc,
-{
-    #[apply(cached_lambda)]
-    #[key(circ.info().hash, HashBytes)]
-    fn recurse(circ: CircuitRc) -> CircuitRc {
-        let inner_mapped = circ.map_children_unwrap(&mut recurse).rc();
-        f(inner_mapped)
-    }
-    recurse(circuit)
-}
-
-/// this applies the function to parent before children, as opposed to deep_map_unwrap, which is children first
-pub fn deep_map_unwrap_preorder<F>(circuit: CircuitRc, f: F) -> CircuitRc
-where
-    F: Fn(CircuitRc) -> CircuitRc,
-{
-    #[apply(cached_lambda)]
-    #[key(circ.info().hash, HashBytes)]
-    fn recurse(circ: CircuitRc) -> CircuitRc {
-        f(circ).map_children_unwrap(&mut recurse).rc()
-    }
-    recurse(circuit)
-}
-
-/// visits each subcircuit that's in any circuit once! (pre-order traversal if it's a tree)
-pub fn visit_circuit<F>(circuit: CircuitRc, mut f: F)
-where
-    F: FnMut(CircuitRc),
-{
-    let mut seen: HashSet<HashBytes> = HashSet::new();
-
-    fn recurse<F>(circ: CircuitRc, seen: &mut HashSet<HashBytes>, f: &mut F)
-    where
-        F: FnMut(CircuitRc),
-    {
-        if !seen.contains(&circ.info().hash) {
-            seen.insert(circ.info().hash);
-            f(circ.clone());
-            for child in circ.children() {
-                recurse(child, seen, f)
-            }
-        }
-    }
-    recurse(circuit, &mut seen, &mut f);
-}
-
 pub fn visit_circuit_with_parents<F>(circuit: CircuitRc, mut f: F)
 where
     F: FnMut(CircuitRc, &Vec<CircuitRc>),
@@ -1431,15 +1399,22 @@ where
 /// does not visit children of circuits where f fails. It does visit all children even if one fails
 /// even though this is more work than stopping on the first child that fails
 /// because it's semantically cleaner to not have to think about which children are first
-pub fn visit_circuit_fallable<F, E>(circuit: CircuitRc, mut f: F) -> Result<(), E>
+#[apply(pycallable)]
+#[pyo3(name = "visit_circuit")]
+pub fn visit_circuit<F>(circuit: CircuitRc, mut f: F) -> Result<(), CircuitConstructionError>
 where
-    F: FnMut(CircuitRc) -> Result<(), E>,
+    F: FnMut((circuit, CircuitRc)) -> Result<(), _>,
 {
+    let mut f = f;
     let mut seen: HashSet<HashBytes> = HashSet::new();
 
-    fn recurse<F, E>(circ: CircuitRc, seen: &mut HashSet<HashBytes>, f: &mut F) -> Result<(), E>
+    fn recurse<F, CircuitConstructionError>(
+        circ: CircuitRc,
+        seen: &mut HashSet<HashBytes>,
+        f: &mut F,
+    ) -> Result<(), CircuitConstructionError>
     where
-        F: FnMut(CircuitRc) -> Result<(), E>,
+        F: FnMut(CircuitRc) -> Result<(), CircuitConstructionError>,
     {
         if !seen.contains(&circ.info().hash) {
             seen.insert(circ.info().hash);
@@ -1756,18 +1731,6 @@ fn use_rust_comp<T: PartialOrd>(l: &T, r: &T, comp_op: CompareOp) -> bool {
     }
 }
 
-fn python_apply_fn_to_sub<N: CircuitNode>(
-    node: &N,
-    f: PyObject,
-) -> Result<N, CircuitConstructionError> {
-    node.map_children(|c| {
-        Python::with_gil(|py| -> Result<_, CircuitConstructionError> {
-            let out = f.call1(py, (c.clone(),))?.extract::<CircuitRc>(py)?;
-            Ok(out)
-        })
-    })
-}
-
 #[derive(Clone, Debug, FromPyObject)]
 enum PyReductionAxes {
     Single(i64),
@@ -1823,6 +1786,10 @@ impl PyCircuitBase {
         use_rust_comp(&self.0, &object.0, comp_op)
     }
 
+    fn __repr__(&self) -> String {
+        self.compiler_repr(true, true)
+    }
+
     #[getter]
     fn hash(&self) -> PyHashBytes {
         PyHashBytes(self.info().hash)
@@ -1856,11 +1823,6 @@ impl PyCircuitBase {
     pub fn print_stats(&self) {
         print_circuit_stats(&self.0)
     }
-    // TODO: rename for rust consistency?
-    pub fn apply_fn_to_sub(&self, f: PyObject) -> PyResult<Circuit> {
-        let out = python_apply_fn_to_sub(&*self.0, f)?;
-        Ok(out)
-    }
 
     #[args(bijection = "true", shape_only_necessary = "false")]
     fn compiler_repr(&self, bijection: bool, shape_only_necessary: bool) -> String {
@@ -1894,6 +1856,31 @@ impl PyCircuitBase {
         evaluate_fn_dtype_device((***self).clone().rc(), device_dtype)
     }
 
+    fn map_children_enumerate(&self, f: PyObject) -> Result<Circuit, CircuitConstructionError> {
+        (&**self)
+            .map_children_enumerate(|i, child| pycall!(f, (i, child), CircuitConstructionError))
+    }
+
+    fn map_children(&self, f: PyObject) -> Result<Circuit, CircuitConstructionError> {
+        (&**self).map_children(|child| pycall!(f, (child,), CircuitConstructionError))
+    }
+
+    fn total_arrayconstant_size(&self) -> BigUint {
+        total_arrayconstant_size(CircuitRc((**self).clone()))
+    }
+
+    fn get_compatible_device_dtype(&self) -> TorchDeviceDtype {
+        get_compatible_dtype(&***self)
+    }
+
+    fn rename(&self, name: Option<String>) -> CircuitRc {
+        (***self).clone().rename(name).rc()
+    }
+
+    fn visit(&self, f: PyObject) -> Result<(), CircuitConstructionError> {
+        visit_circuit_py(CircuitRc((**self).clone()), f)
+    }
+
     fn reduce(
         &self,
         op_name: String,
@@ -1976,7 +1963,6 @@ impl PyCircuitBase {
     }
 }
 
-// TODO: prelude!
 pub fn get_compatible_dtype(circ: &Circuit) -> TorchDeviceDtype {
     circ.info().device_dtype.clone().unwrap_or_defaults()
 }
