@@ -4,24 +4,19 @@ use std::collections::BTreeMap;
 
 use uuid::Uuid;
 
-use super::circuit_node_private::CircuitNodePrivate;
 use super::circuit_utils::deep_map_pass_down_branching;
 use super::{deep_map_unwrap, prelude::*, visit_circuit_with_parents_fallible, NamedAxes};
 
 use crate::pyo3_prelude::*;
 
-pub fn set_named_axes<T: CircuitNode + Clone>(node: &T, named_axes: NamedAxes) -> T {
-    let mut result: T = node.clone();
-    result.info_mut().named_axes = named_axes.clone();
-    let result = result.init_info().unwrap();
-    assert!(result.info().named_axes == named_axes);
-    result
+pub fn set_named_axes<T: CircuitNode>(node: T, named_axes: NamedAxes) -> T {
+    node.update_info(|i| i.named_axes = named_axes).unwrap()
 }
 
 #[pyfunction]
 #[pyo3(name = "set_named_axes")]
 pub fn set_named_axes_py(circuit: CircuitRc, named_axes: NamedAxes) -> CircuitRc {
-    set_named_axes(&**circuit, named_axes).rc()
+    set_named_axes(circuit, named_axes)
 }
 pub fn merge_named_axes(a: &NamedAxes, b: &NamedAxes) -> NamedAxes {
     let mut result = a.clone();
@@ -77,7 +72,7 @@ pub fn propagate_named_axes(
         },
         |circuit, named_axes, children| {
             set_named_axes(
-                &circuit.map_children_unwrap_idxs(|i| children[i].clone()),
+                circuit.map_children_unwrap_idxs(|i| children[i].clone()),
                 merge_named_axes(&circuit.info().named_axes, named_axes),
             )
             .rc()
@@ -165,7 +160,6 @@ pub fn get_axis_leaves(
 /// remove all instances of this axis name from circuit
 pub fn deep_strip_axis_names(circuit: CircuitRc, names: &Option<HashSet<String>>) -> CircuitRc {
     deep_map_unwrap(circuit, |x| {
-        let mut result = (**x).clone();
         let axis_names: NamedAxes = if let Some(blacklist) = &names {
             x.info()
                 .named_axes
@@ -176,7 +170,6 @@ pub fn deep_strip_axis_names(circuit: CircuitRc, names: &Option<HashSet<String>>
         } else {
             BTreeMap::new()
         };
-        result.info_mut().named_axes = axis_names;
-        result.init_info().unwrap().rc()
+        set_named_axes(x, axis_names)
     })
 }
