@@ -20,7 +20,7 @@ use super::{
     prelude::*,
     print::oom_fmt,
     scheduling_z3::schedule_dag_strategy_ints,
-    visit_circuit, Add, ArrayConstant, HashBytes, IrreducibleNode, Leaf, ModuleNodeSpec,
+    visit_circuit_unwrap, Add, ArrayConstant, HashBytes, IrreducibleNode, Leaf, ModuleNodeSpec,
     ScalarConstant, Symbol,
 };
 use crate::pyo3_prelude::*;
@@ -688,7 +688,7 @@ pub fn circuits_to_dag(circuits: &[CircuitRc]) -> Dag {
         result.node_hashes.len() - 1
     };
     for circuit in circuits {
-        visit_circuit(circuit.clone(), &mut |c: CircuitRc| {
+        visit_circuit_unwrap(circuit.clone(), |c: CircuitRc| {
             // ArrayConstants are never added to the dag to begin with because they're always 0 cost
             if !Leaf::matches(&c) {
                 let my_number = number_node(c.clone(), &mut result);
