@@ -46,6 +46,7 @@ pub mod hashmaps;
 pub mod lazy;
 pub mod opt_einsum;
 pub mod py_types;
+pub mod python_callables;
 pub mod rearrange_spec;
 pub mod rrfs;
 pub mod set_cover;
@@ -73,8 +74,7 @@ fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
     };
     use circuit::canonicalize::{canonicalize_node_py, deep_canonicalize_py};
     use circuit::circuit_manipulation::{
-        deep_map_preorder_py, deep_map_py, filter_nodes_py, path_get, replace_nodes_py,
-        update_nodes_py, update_path_py,
+        filter_nodes_py, path_get, replace_nodes_py, update_nodes_py, update_path_py,
     };
     use circuit::circuit_optimizer::{optimize_and_evaluate, optimize_and_evaluate_many};
     use circuit::circuit_utils::{
@@ -284,13 +284,11 @@ fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
         m
     )?)?;
 
-    m.add_function(wrap_pyfunction!(
-        circuit::print::rust_expression_notation_circuit,
-        m
-    )?)?;
+    m.add_function(wrap_pyfunction!(circuit::deep_map_preorder_py, m)?)?;
+    m.add_function(wrap_pyfunction!(circuit::deep_map_py, m)?)?;
+
+    m.add_function(wrap_pyfunction!(circuit::visit_circuit_py, m)?)?;
 
-    m.add_function(wrap_pyfunction!(deep_map_preorder_py, m)?)?;
-    m.add_function(wrap_pyfunction!(deep_map_py, m)?)?;
     m.add_function(wrap_pyfunction!(filter_nodes_py, m)?)?;
     m.add_function(wrap_pyfunction!(replace_nodes_py, m)?)?;
     m.add_function(wrap_pyfunction!(update_nodes_py, m)?)?;
