@@ -3,7 +3,7 @@
 import atexit
 import functools
 import hashlib
-from typing import Tuple
+from typing import Dict, Tuple
 import numpy as np
 import torch
 
@@ -117,6 +117,19 @@ def int_tuple_to_bytes(s: Tuple[int, ...], b):
     return b
 
 
+# import weakref
+# tensor_hash_cache:weakref.WeakKeyDictionary = weakref.WeakKeyDictionary()
+
+# # we had issues with caching by id(tensor), using weakref instead
+# def hash_tensor(tensor):
+#     cached = tensor_hash_cache.get(tensor)
+#     if cached is not None:
+#         return cached
+#     hash_out = hash_tensor_impl(tensor)
+#     tensor_hash_cache[tensor] = hash_out
+#     return hash_out
+
+
 def hash_tensor(tensor):
     m = hashlib.blake2b()
     m.update(b"c3c5ce3e-7d06-4939-8340-2a7e6b2984b4")
